package edu.uoc.android.tfm_magp_2019.activities;

import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.Switch;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.android.tfm_magp_2019.R;
import edu.uoc.android.tfm_magp_2019.manager.FirebaseFenomenoManager;
import edu.uoc.android.tfm_magp_2019.manager.MyDBHandler;
import edu.uoc.android.tfm_magp_2019.utils.Alertas;
import edu.uoc.android.tfm_magp_2019.utils.Fenomenos;

public class AjustesEclipsesActivity extends AppCompatActivity implements CompoundButton.OnCheckedChangeListener {

    private ArrayList eclipsesFeno;
    private Fenomenos fenoRow;
    String id_fenomeno;
    String subTipo;

    Switch solarTotal;
    Switch solarParcial;
    Switch solarAnular;
    Switch lunarTotal;
    Switch lunarParcial;
    Switch lunarPenumbral;

    String literal_ST = "Eclipse Solar Total";
    String literal_SP = "Eclipse Solar Parcial";
    String literal_SA = "Eclipse Solar Anular";
    String literal_LT = "Eclipse Lunar Total";
    String literal_LP = "Eclipse Lunar Parcial";
    String literal_LPe = "Eclipse Lunar Penumbral";

/*
    String literal_ST = getString(R.string.eclipse_solar_total);
    String literal_SP = getString(R.string.eclipse_solar_parcial);
    String literal_SA = getString(R.string.eclipse_solar_anular);
    String literal_LT = getString(R.string.eclipse_lunar_total);
    String literal_LP = getString(R.string.eclipse_lunar_parcial);
    String literal_LPe = getString(R.string.eclipse_lunar_penumbral);
*/
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate( savedInstanceState );
        setContentView( R.layout.activity_ajustes_eclipses );
        Toolbar toolbar = (Toolbar) findViewById( R.id.toolbar );
        setSupportActionBar( toolbar );

        // inicialización de controladores de switchs
        solarTotal = (Switch) findViewById( R.id.switchSolarTotal );
        solarParcial = (Switch) findViewById( R.id.switchSolarParcial );
        solarAnular = (Switch) findViewById( R.id.switchSolarAnular );
        lunarTotal = (Switch) findViewById( R.id.switchLunarTotal );
        lunarParcial = (Switch) findViewById( R.id.switchLunarParcial );
        lunarPenumbral = (Switch) findViewById( R.id.switchLunarPenumbral );

        // asociaciación de listeners al cambio de estado de switchs
        solarParcial.setOnCheckedChangeListener( this );
        solarTotal.setOnCheckedChangeListener( this );
        solarAnular.setOnCheckedChangeListener( this );
        lunarTotal.setOnCheckedChangeListener( this );
        lunarParcial.setOnCheckedChangeListener( this );
        lunarPenumbral.setOnCheckedChangeListener( this );

        // Recupera todos los datos de fenomenos
        final List<Fenomenos> fenomenosList = FirebaseFenomenoManager.getInstance().getAllFenomenos();
        eclipsesFeno = new ArrayList();

        //filtra los datos para recuperar los de tipo eclipse
        for (Fenomenos fenomeno : fenomenosList) {
            if (fenomeno.getTipoFenomeno().equals( getString( R.string.eclipse ) )) {
                eclipsesFeno.add( fenomeno );
            }
        }

        // consulta si está activa alguna de las alertas de la ventana
        findAlertaInicio( solarTotal, literal_ST );
        findAlertaInicio( solarParcial, literal_SP );
        findAlertaInicio( solarAnular, literal_SA );
        findAlertaInicio( lunarTotal, literal_LT);
        findAlertaInicio( lunarParcial, literal_LP );
        findAlertaInicio( lunarPenumbral, literal_LPe);
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

        switch (buttonView.getId()) {
            case R.id.switchSolarTotal:
                funcionalidadSwitchs (buttonView, isChecked, literal_ST, solarTotal, 1);
                break;
            case R.id.switchSolarParcial:
                funcionalidadSwitchs (buttonView, isChecked, literal_SP, solarParcial, 2);
                break;
            case R.id.switchSolarAnular:
                funcionalidadSwitchs (buttonView, isChecked, literal_SA, solarAnular, 3);
                break;
            case R.id.switchLunarTotal:
                funcionalidadSwitchs (buttonView, isChecked, literal_LT, lunarTotal, 4);
                break;
            case R.id.switchLunarParcial:
                funcionalidadSwitchs (buttonView, isChecked, literal_LP, lunarParcial, 5);
                break;
            case R.id.switchLunarPenumbral:
                funcionalidadSwitchs (buttonView, isChecked, literal_LPe, lunarPenumbral,6);
                break;
            default:
                break;
        }
    }

    // busca una alerta por id. retorna true si hay resultados y false en caso contratio
    public boolean findHandlerId(View view) {
        MyDBHandler dbHandler = new MyDBHandler( this, null, null, 1 );
        Alertas alertas = dbHandler.findHandlerId( id_fenomeno );

        if (alertas != null) {
            return true;

        } else {
            return false;
        }
    }

    //busca una alerta por subtipo, retorna true si hay resultados y false en caso contratio
    public boolean findAlertaSubtipo(View view) {
        MyDBHandler dbHandler = new MyDBHandler( this, null, null, 1 );
        Alertas alertas = dbHandler.findHandlerSubtipo( subTipo );

        if (alertas != null) {
            return true;
        } else {
            return false;
        }
    }

    public void addAlerta(View view) {
        MyDBHandler dbHandler = new MyDBHandler( this, null, null, 1 );

        Alertas alertas = new Alertas( Integer.parseInt( id_fenomeno ), "" );
        dbHandler.addHandler( alertas );

    }

    public void addAlertaNombre(View view, int numID) {
        MyDBHandler dbHandler = new MyDBHandler( this, null, null, 1 );

        Alertas alertas = new Alertas( numID, subTipo );
        dbHandler.addHandler( alertas );

    }


    public boolean deleteAlertaId(View view, String numID) {
        MyDBHandler dbHandler = new MyDBHandler( this, null,
                null, 1 );
        boolean result = dbHandler.deleteHandlerId( numID );
        return result;
    }

    // chequea si existe una alerta activa para un tipo de eclipse
    public void findAlertaInicio(Switch switch_eclipse, String sub_tipo) {
        MyDBHandler dbHandler = new MyDBHandler( this, null, null, 1 );
        Alertas alertas = dbHandler.findHandlerSubtipo( sub_tipo );

        //si encuentra resultados pone el check activo
        if (alertas != null) {
            switch_eclipse.setChecked( true );
        } else {
            switch_eclipse.setChecked( false );
        }
    }

    // funcionalidad de cambio de estado de un switch
    public void funcionalidadSwitchs(CompoundButton buttonView, boolean isChecked, String subTipo_ref, Switch switch_ref, int numID){

        boolean borratodas = true;

        //Necesitamos recuperar todos los datos asociados al switch pulsado, por lo que recorremos el array de eclipse hasta encontrar el que se ha pulsado y lo guardamos
        for (int i = 0; i < eclipsesFeno.size(); i++) {
            fenoRow = (Fenomenos) eclipsesFeno.get( i );
            id_fenomeno = fenoRow.getIdFenomeno();
            subTipo = fenoRow.getSubTipo();

            // solo ejecutaremos la logica para el switch seleccionado
            if (subTipo.equals( subTipo_ref )) {
                //buscar que no esté de alta
                if (isChecked) {
                    Boolean result = findHandlerId( buttonView );
                    Boolean resultSubtipo = findAlertaSubtipo( buttonView );
                    //si no está de alta la alerta la damos de alta
                    if (result == false) {
                        addAlerta( buttonView );
                    }
                    // anadimos el subtipo como activo para tener el control de activación
                    if (!resultSubtipo) {
                        addAlertaNombre( buttonView , numID);
                    }
                    //activamos el check
                    switch_ref.setChecked( true );
                } else {
                    //borramos la alerta
                    boolean result = deleteAlertaId( buttonView, id_fenomeno );
                    deleteAlertaId( buttonView, String.valueOf( numID ) );

                    // ponemos el check desctivado
                    switch_ref.setChecked( false );

                    //comprobamos el resultado de la ejecución, si dio KO cambiarmos el booleano borratodas a false para que vuelva a cambiar el check
                    if (!result) {
                        borratodas = false;
                    }
                }
            }
        }
        // comprobamos si en necesario cambiar el check
        if (!borratodas) {
            switch_ref.setChecked( true );
        }
    }

}
