package edu.uoc.android.tfm_magp_2019.fragments;

import android.app.DatePickerDialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import edu.uoc.android.tfm_magp_2019.R;
import edu.uoc.android.tfm_magp_2019.activities.EclipseActivity;
import edu.uoc.android.tfm_magp_2019.adapters.LevelAdapter;
import edu.uoc.android.tfm_magp_2019.manager.FirebaseFenomenoManager;
import edu.uoc.android.tfm_magp_2019.utils.Fenomenos;


public class EclipseFragment extends Fragment implements View.OnClickListener{

    private static final String TAG = "EclipseFragment";

    private static final String CERO = "0";
    private static final String BARRA = "/";

    //Calendario para obtener fecha & hora
    public final Calendar c = Calendar.getInstance();

    //Variables para obtener la fecha
    final int mes = c.get(Calendar.MONTH);
    final int dia = c.get(Calendar.DAY_OF_MONTH);
    final int anio = c.get(Calendar.YEAR);

    //Widgets
    private EditText etFechaDesde;
    private EditText etFechaHasta;
    private Button botonFiltro;
    private ArrayList eclipsesFeno;
    private ArrayList eclipsesFenoFiltradp;
    private Fenomenos fenoRow;
    private LevelAdapter adp;
    private ListView listView;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate( R.layout.eclipsefragment, container, false );

        // inicializamos la listView
        listView = (ListView) view.findViewById( R.id.eclipseList );

        // Recuperamos todos los fenomenos descargados del servidor
        final List<Fenomenos> fenomenosList = FirebaseFenomenoManager.getInstance().getAllFenomenos();
        eclipsesFeno = new ArrayList();

        // filtramos el listado par recuperar unicamente los de tipo eclipse
        for (Fenomenos fenomeno : fenomenosList) {
            if (fenomeno.getTipoFenomeno().equals( getString( R.string.eclipse ) )){
                eclipsesFeno.add( fenomeno );
            }
        }

        // pasamos la información filtrada al adapter para que cargue la listView
        adp=new LevelAdapter(getContext(), R.layout.itemlist_eclipse, eclipsesFeno);
        listView.setAdapter(adp);

        // configuramos las acciones a realizar al pulsar un elemento de la listView
        listView.setOnItemClickListener( new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {

            // Usaremos position para recoger los valores del eclipse del array que tengamos y lo pasaremos con intent a la nueva activity
            fenoRow = (Fenomenos) eclipsesFeno.get( position );

            Intent myIntent = new Intent( view.getContext(), EclipseActivity.class );
            myIntent.putExtra("tipo", fenoRow.getTipoFenomeno());
            myIntent.putExtra("subtipo", fenoRow.getSubTipo());
            myIntent.putExtra("fecha", fenoRow.getFecha());
            myIntent.putExtra("id", fenoRow.getIdFenomeno());
            myIntent.putExtra("latitud", fenoRow.getVisibilidad().getLatitud());
            myIntent.putExtra("longitud", fenoRow.getVisibilidad().getLongitud());
            startActivityForResult( myIntent, 0 );
            }
        } );

        //Widget EditText donde se ejecutará DatePickerDialog para seleccionar una fecha desde y  donde se mostrara la fecha obtenida
        etFechaDesde = (EditText) view.findViewById(R.id.et_mostrar_fecha_picker);
        etFechaDesde.setOnClickListener(this);

        //Widget EditText donde se ejecutará DatePickerDialog para seleccionar una fecha hasta y  donde se mostrara la fecha obtenida
        etFechaHasta = (EditText) view.findViewById(R.id.et_mostrar_fecha_picker2);
        etFechaHasta.setOnClickListener(this);

        // Button de sentencia de filtrado
        botonFiltro = (Button) view.findViewById(R.id.button2);
        botonFiltro.setOnClickListener(this);


        return  view;
    }


    @Override
    public void onClick(View v) {
        switch (v.getId()){
            case R.id.et_mostrar_fecha_picker:
                obtenerFechaDesde();
                break;
            case R.id.et_mostrar_fecha_picker2:
                obtenerFechaHasta();
                break;
            case R.id.button2:
                filtrarEclipses();
                break;
            default:
                break;
        }
    }

    //funcionalidad botón de filtrado. Comprobará que las fechas seleccionadas son validas antes de hacer el filtrado
    private void filtrarEclipses() {

        // comprobar que las dos fechas están informadas
        if (etFechaHasta.getText().toString().isEmpty() || etFechaDesde.getText().toString().isEmpty()) {
            Toast mensajeErrorFechas1 =
                    Toast.makeText(getContext(),
                            "Faltan fechas por informar", Toast.LENGTH_SHORT);
            mensajeErrorFechas1.show();

        }else {
            //cortamos en substrings la fecha para recuperar el día, mes y año
            String diaDesde = etFechaDesde.getText().toString().substring( 0, 2 );
            String mesDesde = etFechaDesde.getText().toString().substring( 3, 5 );
            String anoDesde = etFechaDesde.getText().toString().substring( 6, 10 );
            String diaHasta = etFechaHasta.getText().toString().substring( 0, 2 );
            String mesHasta = etFechaHasta.getText().toString().substring( 3, 5 );
            String anoHasta = etFechaHasta.getText().toString().substring( 6, 10 );

            //pasamos a int los strings
            Integer diaDesdeInteger = Integer.parseInt( diaDesde );
            Integer mesDesdeInteger = Integer.parseInt( mesDesde );
            Integer anoDesdeInteger = Integer.parseInt( anoDesde );
            Integer diaHastaInteger = Integer.parseInt( diaHasta );
            Integer mesHastaInteger = Integer.parseInt( mesHasta );
            Integer anoHastaInteger = Integer.parseInt( anoHasta );

            Toast mensajeErrorFechas = Toast.makeText(getContext(), getString( R.string.rango_fecha_invalido ), Toast.LENGTH_SHORT);

            if (anoDesdeInteger >= anoHastaInteger) {
                if (anoDesdeInteger.equals( anoHastaInteger )) {
                    if (mesDesdeInteger >= mesHastaInteger) {
                        if (mesDesdeInteger.equals( mesHastaInteger )) {
                            if (diaDesdeInteger <= diaHastaInteger){
                                // fechas correctas. filtramos
                                filtradoDeEclipses( anoDesde, mesDesde, diaDesde, anoHasta, mesHasta, diaHasta );

                            }else {
                                // el día desde es mayor que el día hasta, mostramos toast de aviso
                                mensajeErrorFechas.show();
                            }
                        }else {
                            // el mes desde es mayor que el mes hasta, mostramos toast de aviso
                            mensajeErrorFechas.show();
                        }
                    }else{
                        // fechas correctas. filtramos
                        filtradoDeEclipses( anoDesde, mesDesde, diaDesde, anoHasta, mesHasta, diaHasta );
                    }
                } else {
                    // el año desde es mayor que el año hasta, mostramos toas de aviso
                    mensajeErrorFechas.show();
                }
            }else{
                // fechas correctas. filtramos
                filtradoDeEclipses( anoDesde, mesDesde, diaDesde, anoHasta, mesHasta, diaHasta );
            }
        }
    }

    // función de filtrado de eclipses, recupera los eclipses comprendidos entre 2 fechas
    private void filtradoDeEclipses(String anoDesde, String mesDesde, String diaDesde, String anoHasta, String mesHasta, String diaHasta){

        // preparamos las variables de fechas como enteros y en formato yyyymmdd
        String filtroDesde = String.valueOf( anoDesde + mesDesde + diaDesde );
        String filtroHasta = String.valueOf( anoHasta + mesHasta + diaHasta );
        Integer filtroDesdeInteger = Integer.parseInt( filtroDesde );
        Integer filtroHastaInteger = Integer.parseInt( filtroHasta );

        eclipsesFenoFiltradp = new ArrayList();

        // recorremos el listado de eclipses buscando los que cumplan las condiciones de fecha
        for (int i = 0; i < eclipsesFeno.size(); i++) {
            fenoRow = (Fenomenos) eclipsesFeno.get( i );
            String fec_eclipse = fenoRow.getFecha();
            String fec_eclipse_formateada = String.valueOf( fec_eclipse.substring( 4,8 ) + fec_eclipse.substring( 2,4 ) + fec_eclipse.substring( 0,2 ) );
            Integer fec_ecl_forma_Integer = Integer.parseInt( fec_eclipse_formateada );
            // si la fecha del eclipse está comprendida entre los filtros añado el eclipse al listado de resultados
            if (fec_ecl_forma_Integer >= filtroDesdeInteger && fec_ecl_forma_Integer <= filtroHastaInteger ) {

                eclipsesFenoFiltradp.add( fenoRow );

            }
        }

        // pasamos el listado al adaptar para que actualice el listado en ventana
        adp=new LevelAdapter(getContext(), R.layout.itemlist_eclipse, eclipsesFenoFiltradp);
        listView.setAdapter(adp);
    }


    private void obtenerFechaDesde(){
        DatePickerDialog recogerFecha = new DatePickerDialog(getContext(), new DatePickerDialog.OnDateSetListener() {
            @Override
            public void onDateSet(DatePicker view, int year, int month, int dayOfMonth) {
                //Esta variable lo que realiza es aumentar en uno el mes ya que comienza desde 0 = enero
                final int mesActual = month + 1;
                //Formateo el día obtenido: antepone el 0 si son menores de 10
                String diaFormateado = (dayOfMonth < 10)? CERO + String.valueOf(dayOfMonth):String.valueOf(dayOfMonth);
                //Formateo el mes obtenido: antepone el 0 si son menores de 10
                String mesFormateado = (mesActual < 10)? CERO + String.valueOf(mesActual):String.valueOf(mesActual);
                //Muestro la fecha con el formato deseado
                etFechaDesde.setText(diaFormateado + BARRA + mesFormateado + BARRA + year);
            }
            //Estos valores deben ir en ese orden, de lo contrario no mostrara la fecha actual
            /**
             *También puede cargar los valores que usted desee
             */
        },anio, mes, dia);

        //Muestro el widget
        recogerFecha.show();

    }

    private void obtenerFechaHasta(){
        DatePickerDialog recogerFecha = new DatePickerDialog(getContext(), new DatePickerDialog.OnDateSetListener() {
        //DatePickerDialog recogerFecha = new DatePickerDialog(getContext(), R.style.TimePickerTheme, new DatePickerDialog.OnDateSetListener(){
            @Override
            public void onDateSet(DatePicker view, int year, int month, int dayOfMonth) {
                //Esta variable lo que realiza es aumentar en uno el mes ya que comienza desde 0 = enero
                final int mesActual = month + 1;
                //Formateo el día obtenido: antepone el 0 si son menores de 10
                String diaFormateado = (dayOfMonth < 10)? CERO + String.valueOf(dayOfMonth):String.valueOf(dayOfMonth);
                //Formateo el mes obtenido: antepone el 0 si son menores de 10
                String mesFormateado = (mesActual < 10)? CERO + String.valueOf(mesActual):String.valueOf(mesActual);
                //Muestro la fecha con el formato deseado
                etFechaHasta.setText(diaFormateado + BARRA + mesFormateado + BARRA + year);
            }
            //Estos valores deben ir en ese orden, de lo contrario no mostrara la fecha actual
            /**
             *También puede cargar los valores que usted desee
             */
        },anio, mes, dia);

        //Muestro el widget
        recogerFecha.show();

    }

}
