package edu.uoc.android.tfm_magp_2019.manager;

import android.util.Log;

import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import edu.uoc.android.tfm_magp_2019.utils.Fenomenos;

public class FirebaseFenomenoManager {

    private static final String FENOMENOS_URL = "https://tfmmggp2019.firebaseio.com/";

    // Class instance
    private static FirebaseFenomenoManager instance = null;
    // Reference of Firebase object
    private DatabaseReference firebaseFenomenoRef;
    // List of Contacts
    private HashMap<String, Fenomenos> fenomenosHashMap = new HashMap<>();

    /**
     * Classic Singleton design pattern implementation
     */
    public static FirebaseFenomenoManager getInstance() {
        Log.d( "Traza servidor - ", "FirebaseFenomenoManager - getInstance 1" );
        if (instance == null) {
            Log.d( "Traza servidor - ", "FirebaseFenomenoManager - getInstance 2" );
            instance = new FirebaseFenomenoManager();
        }
        return instance;
    }

    private FirebaseFenomenoManager() {
        // Initialize Firebase reference
        Log.d( "Traza servidor - ", "FirebaseFenomenoManager - FirebaseFenomenoManager 1" );
        firebaseFenomenoRef = FirebaseDatabase.getInstance().getReferenceFromUrl(FENOMENOS_URL);
    }

    /**
     * Retrieving the list of edu.uoc.android.contacts from Firebase server
     */
    public void getFenomenosFromServer(ValueEventListener listener) {
        Log.d( "Traza servidor - ", "FirebaseFenomenoManager - getFenomenosFromServer 1" );
        firebaseFenomenoRef.keepSynced(true);
        Log.d( "Traza servidor - ", "FirebaseFenomenoManager - getFenomenosFromServer 2" );
        Query queryRef = firebaseFenomenoRef.orderByKey();
        Log.d( "Traza servidor - ", "FirebaseFenomenoManager - getFenomenosFromServer 3" );
        queryRef.addListenerForSingleValueEvent(listener);
    }

    /**
     * Retrieving the list of edu.uoc.android.contacts
     */
    public List<Fenomenos> getAllFenomenos() {

        Log.d( "Traza servidor - ", "FirebaseFenomenoManager - getAllFenomenos 1" );
        return new ArrayList<>(fenomenosHashMap.values());
    }

    /**
     * Retrieving a contact from list
     */
    public Fenomenos getFenomenosByObjectId(String objectId) {
        Log.d( "Traza servidor - ", "FirebaseFenomenoManager - getContactByObjectId 1" );
        return fenomenosHashMap.get(objectId);
    }

    /**
     * Adding a contact to HashMap
     */
    public void addfenomenosHashMap(Fenomenos fenomeno) {
        Log.d( "Traza servidor - ", "FirebaseFenomenoManager - addfenomenosHashMap 1" );
        this.fenomenosHashMap.put(fenomeno.getIdFenomeno(), fenomeno);
    }
}